#! python

# ##############################################################################
# Copyright (c) 2014, Stanley Engle
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, 
#    this list of conditions and the following disclaimer.

# 2. Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# ##############################################################################

"""Provides functions to download/upload files from/to a FTP server.

The FTP functionality provided in this module is not designed for 
secure FTP transactions.  Also, once the functions complete the file 
transfers, the FTP connection to the server is closed.  If more 
transactions are required, the functions create a new FTP object to 
use to reconnect.

"""
# Standard Library Imports
from ftplib import FTP
import argparse
import os
import os.path


# The version number of this module.
VERSION = '0.0.1'


def download(host, port, user, passwd, remote_filenames = [], 
    local_dir = os.getcwd() + '/', remote_dir = '/', binary = True):
    """Downloads files via FTP from a remote server.

    Parameters
    ----------
    host : str
        Hostname of remote server.
    port : int
        The port on the remote server used by the FTP service.
    user : str
        The username to use to login to the remote server.
    passwd : str
        The user password used to login to the remote server.
    remote_filenames : list
        A list of filenames to download from the remote server.
    local_dir : str
        The local directory where the remote files will be saved.
    remote_dir : str
        The directory on the remote server where the files are located.
    binary : bool
        If True, download in binary mode, otherwise ascii mode is used.

    Returns
    -------
    None

    """
    ftp = FTP()
    gtg = True

    try:

        ftp.connect(host, port)
        ftp.login(user, passwd)
        ftp.cwd(remote_dir)

    except:
        gtg = False

    if gtg is True:

        actual_remote_filenames = ftp.nlst()

        retr = ftp.retrlines
        mode = 'w'
    
        if binary:
    
            retr = ftp.retrbinary
            mode = 'wb'
        
        if len(remote_filenames) == 0:
    
            for remote_filename in actual_remote_filenames:
    
                f = open(local_dir + remote_filename, mode)
                retr('RETR %s' % remote_filename, f.write)
                f.close()
    
        else:

            for remote_filename in remote_filenames:
    
                if remote_filename in actual_remote_filenames:

                    f = open(local_dir + remote_filename, mode)
                    retr('RETR %s' % remote_filename, f.write)
                    f.close()
    
    ftp.close()

    return None


def upload(host, port, user, passwd, local_filenames, remote_dir = '/',  
    binary = True):
    """Upload files via FTP to a remote server.

    Parameters
    ----------
    host : str
        Hostname of remote server.
    port : int
        The port on the remote server used by the FTP service.
    user : str
        The username to use to login to the remote server.
    passwd : str
        The user password used to login to the remote server.
    local_filenames : list
        A list of filenames to upload to the remote server.
    remote_dir : str
        The remote server directory where the uploaded files will be 
        stored.
    binary : bool
        If True, upload in binary mode, otherwise ascii mode is used.

    Returns
    -------
    None

    """
    ftp = FTP()
    gtg = True

    try:

        ftp.connect(host, port)
        ftp.login(user, passwd)
        ftp.cwd(remote_dir)

    except:
        gtg = False

    if gtg is True:

        stor = ftp.storlines
        mode = 'r'
    
        if binary:
    
            stor = ftp.storbinary
            mode = 'rb'
    
        for local_filename in local_filenames:
    
            if os.path.isfile(local_filename) is True:

                basename = os.path.basename(local_filename)
        
                f = open(local_filename, mode)
                stor('STOR %s' % basename, f)
                f.close()

    ftp.close()

    return None


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest = 'subcommand')
    dparser = subparsers.add_parser('download')
    uparser = subparsers.add_parser('upload')

    # The parser arguments.
    parser.add_argument('-p', '--port', type = int, default = 21, 
        dest = 'port', help = 'Server port')
    parser.add_argument('-u', '--user', type = str, default = 'anonymous', 
        dest = 'user', help = 'Remote server user (default: anonymous)')
    parser.add_argument('--passwd', type = str, default = 'anonymous@', 
        dest = 'passwd', help = 'Remote server user password ' + \
            '(default: anonymous@)')
    parser.add_argument('-r', '--remote_directory', type = str, default = '/', 
        dest = 'remote_directory', 
        help = 'Remote directory to use for the file transfer')

    # The subparser arguments for downloading files.
    dparser.add_argument('-l', '--local_directory', type = str, 
        default = os.getcwd() + '/', dest = 'local_directory', 
        help = 'Local directory where files will be downloaded (default: cwd)')
    dparser.add_argument('host', help = 'Hostname of FTP server', nargs = 1)
    dparser.add_argument('filenames', metavar = 'FN', type = str, nargs = '*',
                   help='a name of a file to download')

    # The subparser arguments for uploading files.
    uparser.add_argument('host', help = 'Hostname of FTP server', nargs = 1)
    uparser.add_argument('filenames', metavar = 'FN', type = str, nargs = '+',
                   help='a name of a file to transfer')

    args = parser.parse_args()

    subcommand = args.subcommand
    host = args.host[0]
    port = args.port
    user = args.user
    passwd = args.passwd
    remote_dir = args.remote_directory
    filenames = args.filenames

    if subcommand == 'upload':
        simple_ftp.upload(host, port, user, passwd, filenames, remote_dir)

    elif subcommand == 'download':

        local_dir = args.local_directory
        simple_ftp.download(host, port, user, passwd, filenames, local_dir, 
            remote_dir)

